function addbody(...)
	bodies[#bodies+1] = love.physics.newBody(world, ...)
end
function addpolyshape(body, ...)
	shapes[#shapes+1] = love.physics.newPolygonShape(bodies[body], ...)
	bodyindex[#shapes] = bodies[body]
end
function addrectshape(body, ...)
	shapes[#shapes+1] = love.physics.newRectangleShape(bodies[body], ...)
	bodyindex[#shapes] = bodies[body]
end
function addbox(x, y)
	addbody(x, y, 10, 10)
	addrectshape(#bodies, 0, 0, 20, 20, 0)
end
function addrevjoint(b1, b2, x, y)
	joints[#joints+1] = love.physics.newRevoluteJoint(bodies[b1], bodies[b2], x, y)
end

function love.load()
	world = love.physics.newWorld(-50, -50, 850, 650, 0, 50, false)
	bodies = {}
	shapes = {}
	joints = {}
	bodyindex = {}
	addbody(400, 600, 0, 0)
	addrectshape(1, 0, 0, 800, 25, 0)
	math.randomseed(os.time())
	for i=1,20 do
		addbox(400-math.random(70)+math.random(70), 400+math.random(160))
	end
	addbody(200, 200, 20, 20)
	addrectshape(#bodies, 0, 0, 100, 20, 0)
	addrectshape(#bodies, 0, 0, 20, 100, 0)
	addbody(600, 200, 60, 10)
	addpolyshape(#bodies, -50, -43, 50, -43, 0, 43) 
	addbody(600, 400, 120, 5)
	addrectshape(#bodies, 0, 0, 40, 40, 0)
	for i=1,6 do
		addbody(240+40*i, 100, 5, 5)
		addrectshape(#bodies, 0, 0, 50, 15)
		if i>1 then
			addrevjoint(#bodies-1, #bodies, 220+40*i, 100)
		end
	end
	love.graphics.setLineWidth(2)
end

local speed = 1
function love.update(dt)
	if not love.keyboard.isDown' ' then
		if love.keyboard.isDown'lshift' then
			if speed > .2 then
				speed = speed * .9
			end
		else
			if speed < 1 then
				speed = speed / .9
			end
		end
		world:update(dt*speed)
	end
	local x, y = love.mouse.getPosition()
	if love.mouse.isDown'l' then
		if not sel then
			sel = 0
			dragtime = 0
			for i=1,#shapes do
				if shapes[i]:testPoint(x, y) then
					sel = i
					startx = x
					starty = y
					break
				end
			end
		end
		dragtime = dragtime + dt
	else
		if sel and sel > 0 then
			local b = bodyindex[sel] --shapes[sel]:getBody() -- bug in love
			b:applyImpulse((x - startx)/dragtime*.3, (y - starty)/dragtime*.3, b:getX(), b:getY())
		end
		sel = nil
	end
	if love.mouse.isDown'r' then
		for i=1,#shapes do
			if shapes[i]:testPoint(x, y) then
				bodyindex[i]:applyTorque(1000)
				break
			end
		end
	end
end

local sqrt, max, min = math.sqrt, math.max, math.min
function love.draw()
	for i=1,#shapes do
		love.graphics.polygon('line', shapes[i]:getPoints())
	end
	if sel and sel > 0 then
		local x, y = love.mouse.getPosition()
		local d = sqrt((startx-x)^2 + (starty-y)^2)
		love.graphics.setColor(255,255,255,min(max(255-dragtime*200+d,0),255))
		love.graphics.line(startx, starty, x, y)
		love.graphics.setColor(255,255,255)
	end
end
